import { createEffect, createSignal } from 'solid-js';
import { SetStoreFunction, Store, createStore } from 'solid-js/store';
import { AuthenticationResponse } from './api';
import { DialogInputField, DialogProps } from './components/Dialog';

// Source: https://www.solidjs.com/examples/todos
function createLocalStore<T extends object>(
	name: string,
	init: T
): [Store<T>, SetStoreFunction<T>] {
	const localState = localStorage.getItem(name);
	const [state, setState] = createStore<T>(
		localState ? JSON.parse(localState) : init
	);
	createEffect(() => localStorage.setItem(name, JSON.stringify(state)));
	return [state, setState];
}

export interface LocalState {
	accountInfo: AuthenticationResponse | null,
}

export const [localState, setLocalState] = createLocalStore<LocalState>('localState', { accountInfo: null });

export const [dialogs, setDialogs] = createSignal([] as DialogProps[]);

export const showDialog = (dialog: DialogProps) => {
	const newDialog: DialogProps = {
		...dialog,
		onDismiss: () => {
			setDialogs(ds => {
				const newDs = [...ds];
				let idx = ds.indexOf(newDialog);
				if (idx == -1) return newDs;
				newDs.splice(idx, 1);
				return newDs;
			});
			dialog.onDismiss?.();
		}
	};

	setDialogs([...dialogs(), newDialog]);
};

export const showInputDialog = (title: string, message: string, name: string, callback: (text: string) => void) => {
	const [text, setText] = createSignal('');

	showDialog({
		title,
		text: message,
		inputFields: [{ name, onInput: setText }],
		buttons: [{ name: 'Okay', action: () => callback(text()) }, "cancel"]
	});
}

export const showInputsDialog = async (title: string, message: string, inputFields: DialogInputField[], callback: () => void) => {
	showDialog({
		title,
		text: message,
		inputFields: inputFields,
		buttons: [{ name: 'Okay', action: () => callback() }]
	});
}

export const showMessageDialog = (title: string, text: string) => showDialog({
	title,
	text
});
